# Copyright 2019 Alvaro Bartolome @ alvarobartt in GitHub
# See LICENSE for details.

__author__ = "Alvaro Bartolome @ alvarobartt in GitHub"
__version__ = "0.4"

from pyrtfolio.StockPortfolio import StockPortfolio
